@@ -5,6 +5,7 @@ define(["../notjQuery", "Completer"], function ($, Completer) {
     class BaseInput {
         constructor(input) {
             this.input = input;
+            this.readOnly = false;
             this.disabled = false;
             this.separator = '';
             this.usedTerms = [];
@@ -55,9 +56,12 @@ define(["../notjQuery", "Completer"], function ($, Completer) {
             $(this.termContainer).on('input', '[data-label]', this.onInput, this);
             $(this.termContainer).on('keydown', '[data-label]', this.onKeyDown, this);
             $(this.termContainer).on('keyup', '[data-label]', this.onKeyUp, this);
-            $(this.termContainer).on('focusout', '[data-index]', this.onTermFocusOut, this);
             $(this.termContainer).on('focusin', '[data-index]', this.onTermFocus, this);
 
+            if (! this.readOnly) {
+                $(this.termContainer).on('focusout', '[data-index]', this.onTermFocusOut, this);
+            }
+
             // Copy/Paste
             $(this.input).on('paste', this.onPaste, this);
             $(this.input).on('copy', this.onCopyAndCut, this);
@@ -306,7 +310,8 @@ define(["../notjQuery", "Completer"], function ($, Completer) {
         }
 
         insertRenderedTerm(label) {
-            let next = this.termContainer.querySelector(`[data-index="${ label.dataset.index + 1 }"]`);
+            const termIndex = Number(label.dataset.index);
+            const next = this.termContainer.querySelector(`[data-index="${ termIndex + 1 }"]`);
             this.termContainer.insertBefore(label, next);
             return label;
         }
@@ -460,10 +465,10 @@ define(["../notjQuery", "Completer"], function ($, Completer) {
             // Cut the term's data
             let [termData] = this.usedTerms.splice(termIndex, 1);
 
-            // Avoid saving the term, it's removed after all
-            label.firstChild.skipSaveOnBlur = true;
-
             if (updateDOM) {
+                // Avoid saving the term, it's removed after all
+                label.firstChild.skipSaveOnBlur = true;
+
                 // Remove it from the DOM
                 this.removeRenderedTerm(label);
             }
@@ -664,7 +669,10 @@ define(["../notjQuery", "Completer"], function ($, Completer) {
                 toFocus = this.input;
             }
 
-            toFocus.selectionStart = toFocus.selectionEnd = 0;
+            if (! this.readOnly) {
+                toFocus.selectionStart = toFocus.selectionEnd = 0;
+            }
+
             $(toFocus).focus();
 
             return toFocus;
@@ -687,7 +695,10 @@ define(["../notjQuery", "Completer"], function ($, Completer) {
                 toFocus = this.input;
             }
 
-            toFocus.selectionStart = toFocus.selectionEnd = toFocus.value.length;
+            if (! this.readOnly) {
+                toFocus.selectionStart = toFocus.selectionEnd = toFocus.value.length;
+            }
+
             $(toFocus).focus();
 
             return toFocus;
@@ -801,7 +812,12 @@ define(["../notjQuery", "Completer"], function ($, Completer) {
                 case 'Backspace':
                     removedTerms = this.clearSelectedTerms();
 
-                    if (this.isTermDirectionVertical()) {
+                    if (this.readOnly) {
+                        if (termIndex >= 0) {
+                            removedTerms[termIndex] = this.removeTerm(input.parentNode);
+                            this.moveFocusForward(termIndex - 1);
+                        }
+                    } else if (this.isTermDirectionVertical()) {
                         // pass
                     } else if (termIndex >= 0 && ! input.value) {
                         let removedTerm = this.removeTerm(input.parentNode);
@@ -835,7 +851,12 @@ define(["../notjQuery", "Completer"], function ($, Completer) {
                 case 'Delete':
                     removedTerms = this.clearSelectedTerms();
 
-                    if (! this.isTermDirectionVertical() && termIndex >= 0 && ! input.value) {
+                    if (this.readOnly) {
+                        if (termIndex >= 0) {
+                            removedTerms[termIndex] = this.removeTerm(input.parentNode);
+                            this.moveFocusForward(termIndex - 1);
+                        }
+                    } else if (! this.isTermDirectionVertical() && termIndex >= 0 && ! input.value) {
                         let removedTerm = this.removeTerm(input.parentNode);
                         if (removedTerm !== false) {
                             input = this.moveFocusForward(termIndex - 1);
@@ -863,20 +884,20 @@ define(["../notjQuery", "Completer"], function ($, Completer) {
                     }
                     break;
                 case 'ArrowLeft':
-                    if (input.selectionStart === 0 && this.hasTerms()) {
+                    if (this.hasTerms() && (this.readOnly || input.selectionStart === 0)) {
                         event.preventDefault();
                         this.moveFocusBackward();
                     }
                     break;
                 case 'ArrowRight':
-                    if (input.selectionStart === input.value.length && this.hasTerms()) {
+                    if (this.hasTerms() && (this.readOnly || input.selectionStart === input.value.length)) {
                         event.preventDefault();
                         this.moveFocusForward();
                     }
                     break;
                 case 'ArrowUp':
                     if (this.isTermDirectionVertical()
-                        && input.selectionStart === 0
+                        && (this.readOnly || input.selectionStart === 0)
                         && this.hasTerms()
                         && (this.completer === null || ! this.completer.isBeingCompleted(input))
                     ) {
@@ -886,7 +907,7 @@ define(["../notjQuery", "Completer"], function ($, Completer) {
                     break;
                 case 'ArrowDown':
                     if (this.isTermDirectionVertical()
-                        && input.selectionStart === input.value.length
+                        && (this.readOnly || input.selectionStart === input.value.length)
                         && this.hasTerms()
                         && (this.completer === null || ! this.completer.isBeingCompleted(input))
                     ) {
@@ -974,6 +995,10 @@ define(["../notjQuery", "Completer"], function ($, Completer) {
 
             this.deselectTerms();
 
+            if (this.readOnly) {
+                return;
+            }
+
             if (! this.hasSyntaxError(input) && (
                 this.completer === null || ! this.completer.isBeingCompleted(input, false)
             )) {
